@@ -582,6 +582,11 @@ EXPORTED int conversations_get_msgid(struct conversations_state *state,
  *    we have observed combinations of buggy client software both
  *    add and remove whitespace around folding points.
  *
+ *  - We include the Unicode U+00A0 (non-breaking space) codepoint in our
+ *    determination of whitespace (as the UTF-8 sequence \xC2\xA0) because
+ *    we have seen it in the wild, but do not currently generalise this to
+ *    other Unicode "whitespace" codepoints. (XXX)
+ *
  *  - Because we eliminate whitespace entirely, and whitespace helps
  *    delimit some of our other replacements, we do that whitespace
  *    step last instead of first.
@@ -611,7 +616,7 @@ EXPORTED void conversation_normalise_subject(struct buf *s)
     int r;
 
     if (!initialised_res) {
-        r = regcomp(&whitespace_re, "[ \t\r\n]+", REG_EXTENDED);
+        r = regcomp(&whitespace_re, "([ \t\r\n]+|\xC2\xA0)", REG_EXTENDED);
         assert(r == 0);
         r = regcomp(&relike_token_re, "^[ \t]*[A-Za-z0-9]+(\\[[0-9]+\\])?:", REG_EXTENDED);
         assert(r == 0);
